# Generated by Django 3.0.9 on 2020-08-18 04:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('waffle', '0003_update_strings_for_i18n'),
    ]

    operations = [
        migrations.AlterField(
            model_name='flag',
            name='everyone',
            field=models.BooleanField(blank=True, help_text='Flip this flag on (Yes) or off (No) for everyone, overriding all other settings. Leave as Unknown to use normally.', null=True, verbose_name='Everyone'),
        ),
    ]
